;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; elnw - enhancements for using emacs lisp (and all lisp variants) in
;;        the noweb literate programming environment
;;      - auto-indexing of lisp functions,variables,constants, etc...
;;      - production of font-lock colored source code for all 
;;        programming languages in dvi and pdf literate program
;;        documentation 
;;
;; Copyright (C) 2007  Barry Krofchick
;;
;; This program is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2 of the License, or
;; (at your option) any later version.
;;
;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with this program; if not, write to the Free Software Foundation, Inc.,
;; 51 Franklin Street - Fifth Floor, Boston, MA 02110-1301, USA.
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(require 'cl)
(require 'bytecomp)
(require 'hideshow)
(provide 'elnw)
;;;
;;; we open by defining the dummy function
;;; definitions for the noweb doc elements
;;;
(defun noweb-doc (next-root doc-string)
"This is the dummy function used to insert
documentation into a .el file. NEXT-ROOT is
either a string or symbol whose name is the name
of the code chunk that follows this doc string or nil
if we do not wish to end this doc chunk with
the beginning of a code chunk. DOC-STRING is
the ~LaTeX encoded documentation chunk." t)

(defun noweb-chunk (root &rest chunks)
"Flag a noweb start of code chunk marker. 
ROOT is a string or symbol (whose name) is the
referenced code chunk that follows. The remaining arguments
are any code chunk references (to other chunks) that
are to be inserted at the start of this part of
the ROOT code chunk" t)

(defun noweb-other-code (root code)
"Flag a noweb start of code marker for ROOT.
The code is in a language other than lisp
and is inserted without any translation." t)

;;; this preamble will go into the first
;;; code chunk below - but we need it
;;; at the top to establish definitions
;;; for the function calls below
(noweb-doc nil
"
% flag the programming language for
% subsequent chunks as elisp (elisp-mode)
@@language elisp
% flag that we are using ~ as the tex macro start
% as opposed to \\
@@macro-marker ~

~include{overview}

~section{~emph{elnw.el} Source Code}

The primary source elisp source code, all
contained in file elnw~extel, begins. 


~subsection{Header / Globals}

We start with the dummy
functions for noweb-doc and noweb-chunk. These
functions when executed all do nothing.
Their value is as placeholders for the elnw
converter which transforms the elisp noweb 
code into pure noweb by picking up these function
markers and performing the translation.
")


(noweb-doc "elnw.el"
"
The following constants reference external
programs required and used by this package.
These standard programs must be available in
the PATH.
")
(defconst ELNW-NOROOTS "noroots")
(defconst ELNW-NOTANGLE "notangle")
(defconst ELNW-NOWEAVE "noweave")
(defconst ELNW-LATEX "latex")
(defconst ELNW-PDFLATEX "pdflatex")

(noweb-doc t
"
ELNW-FONT-LOCK-MAP contains the name of
a file that will be generated automatically
by elnw-convert, and stored in the user root
directory, when elnw-convert is run from emacs. 
It picks up the user font-lock face colors and
stores them for use by the fontification filter.

This file may edited manually if desired, to alter
source code coloration, however using emacs customization
is the preferred method.
")

(defconst ELNW-FONT-LOCK-MAP ".noweb-font-lock-colors")

(noweb-doc t
"The following 2 constants define a standard
latex preamble and postfix for auto-doc generation.
These constants may be re-defined in the .~,emacs
file if you need something different.
")

(defconst ELNW-TEX-PREAMBLE 
"
\\documentclass{article}
\\usepackage{noweb}
\\pagestyle{noweb}
\\def\\nwendcode{\\endtrivlist \\endgroup \\vfil\\penalty10\\vfilneg}
\\let\\nwdocspar=\\smallbreak
\\usepackage[showerrors,table,x11names,svgnames,hyperref]{xcolor}
%\\definecolors{Red,CadetBlue,ForestGreen,DarkGoldenrod,Blue,Orchid,Purple,Purple,RosyBrown,FireBrick}
\\noweboptions{smallcode,margintag}
\\usepackage{hyperref}
\\begin{document}
")

(defconst ELNW-TEX-PREAMBLE-XDVI
"
\\documentclass{article}
\\usepackage{noweb}
\\pagestyle{noweb}
\\def\\nwendcode{\\endtrivlist \\endgroup \\vfil\\penalty10\\vfilneg}
\\let\\nwdocspar=\\smallbreak
\\usepackage[showerrors,table,x11names,svgnames,dvips,hyperref]{xcolor}
%\\definecolors{Red,CadetBlue,ForestGreen,DarkGoldenrod,Blue,Orchid,Purple,Purple,RosyBrown,FireBrick}
\\noweboptions{smallcode,margintag}
\\usepackage{hyperref}
\\begin{document}
")


(defconst ELNW-TEX-FINAL "
\\newpage
\\section{INDEX}
\\nowebchunks
\\nowebindex
\\end{document}\n")

(defconst ELNW-TEX-RAWCODE-PREAMBLE-XDVI
"
\\documentclass{article}
\\usepackage{noweb}
\\pagestyle{noweb}
\\def\\nwendcode{\\endtrivlist \\endgroup \\vfil\\penalty10\\vfilneg}
\\let\\nwdocspar=\\smallbreak
\\usepackage[showerrors,table,x11names,svgnames,dvips,hyperref]{xcolor}
%\\definecolors{Red,CadetBlue,ForestGreen,DarkGoldenrod,Blue,Orchid,Purple,Purple,RosyBrown,FireBrick}
\\noweboptions{smallcode,margintag}
\\usepackage{hyperref}
\\begin{document}
\\nwbegincode{1}
")

(defconst ELNW-TEX-RAWCODE-FINAL
"
\\nwendcode{}
\\end{document}
")

(noweb-doc "elnw.el"
"
Whenever we use double-left angle brackets in
the code we reference the following constant
instead. Since these double-left angle brackets
can confuse the noweb code about whether we intend
them as chunk references or other and in this
code - since we are formatting noweb style documents
we need to be able to reference them literally in
the code without quoting them wit a preceding @ sign
as a signal to noweb tools.
") 

(defconst ELNW-ANG-OPEN (concat "<" "<"))

(noweb-doc "elnw.el" 
"
The following constants are used by the elnw
parser in translating .\,nw.\,el files to
.\,nw files by removing the noweb-doc and 
noweb-chunk function calls and turning them
into syntactically correct noweb format
replacements.
")

(defconst NOWEB-DOC "noweb-doc"
"noweb-doc function call marker")
(defconst NOWEB-CHUNK "noweb-chunk"
"noweb-chunk function call marker")
(defconst NOWEB-CODE "noweb-other-code")
(noweb-doc "elnw.el"
"
The ELNW-TEX-MACRO-START constant indicates any
character-string the user wishes to use in place 
of backslash in his doc chunk strings to indicate 
a ~LaTeX macro start character. Backslashes as 
start of tex macros have to be doubled which is
somewhat annoying. Placing a line of
the form @@tex-macro-start string anywhere in
a doc chunk will change the macro start for this
and subsequent chunks to the indicated character
string, allowing the user to select any convenient
start of macro character - other than backslash.")
 
(defconst ELNW-TEX-MACRO-START '("\\" "\\")
"replace occurrences of ~ with back-slash
in noweb-doc strings - avoids problems 
with emacs coming up with bad escape
sequences when compiling doc string with backslashes")


(noweb-doc "elnw.el"
"
The elnw-lang-flags list defines the modenames, default
file extensions and single line comment marker for
the most common languages. When colorizing and
fontifying source code - we need to know the
proper file extension for the language so that
the emacs font-lock can automagically detect the
mode. We also need to know a single line 
comment marker for the language so we can 
drive out @use statements in the source code
which are chunk references as single line
comments and then remove them and reapply them
once fontified as standard @use statements.

To add any new languages, either re-define
this list in your ~~/.~,emacs file or edit
here.
")

(defconst elnw-lang-flags '(("elisp" "el" ";;")
		       ("c" "c" "//")
		       ("c++" "c++" "//")
		       ("tex" "tex" "%")
		       ("sh"  "sh"  "#"))
"elnw-lang-flags is an a-list searched on language name.
Each entry holds the default file type and the single-line
comment marker for the language.")


(noweb-doc "elnw.el"
"
The variable ~emph{elnw-dot-emacs-loaded} is 
set to prevent multiple loadings of the users
.~,emacs file.
")

(defvar elnw-dot-emacs-loaded nil)


(noweb-doc "elnw.el"
"
The following table defines the default
set of colors to use for the various
face types. Change these colors
if you wish to customize for your
environment. We can't really get the
faces directly in our fontification
filter because emacs does not attach the
actual colors for faces until a frame
is created and we are running fontification
filter as a batch process without any frames.
Just modify these colors if they do not
meet your needs. You can use the 
Options / Text Properties / Display Colors
menu to get the color names for all supported
colors.
")

(defvar elnw-default-face-colors '
  ((font-lock-warning-face "Red") 
   (font-lock-constant-face "CadetBlue") 
   (font-lock-type-face "ForestGreen") 
   (font-lock-variable-name-face "DarkGoldenrod") 
   (font-lock-function-name-face "Blue") 
   (font-lock-builtin-face "Orchid") 
   (font-lock-keyword-face "Purple") 
   (font-lock-doc-face "Purple") 
   (font-lock-string-face "RosyBrown") 
   (font-lock-comment-face "FireBrick")))

(defconst elnw-faces-defining-things 
  '(font-lock-variable-name-face
    font-lock-function-name-face)
"These are the faces we use to identify defines
in lisp code")

(noweb-doc "elnw.el"
"
~subsection{Externally Usable Functions}

The functions in this section of the
code are user invokable. They define the
usable function interface for this package.

~subsubsection{~emph{elnw} - converter no args }
The function ~emph{elnw} takes no arguments.
It is the primary entry point for a 
shell script to launch emacs in batch
mode to transform ~extnwel files to
~extnw files by transforming the noweb-doc
and noweb-chunk function calls found.

The shell script sets 2 environment variables
ELIN and ELOUT to set the input and output
filenames. If ELOUT is missing - then the default
uses the ELIN file name stripped of the trailing
~extel extension.

This function simply checks for the correctness
of the arguments and the existence of the input
file and sets up an output filename if required.

If all is OK it calls the function create-noweb-list
to do the actual work. This function is interactive
and hence may be invoked with an alt-x command
from within emacs to initiate the conversion directly.
")

(defun elnw-convert ()
  (let ((infile (getenv "ELIN"))
	(outfile (getenv "ELOUT"))
	(in-root))
    ;; extract infile and outfile
    ;; from ELIN and ELOUT environment variables
    ;; set by calling script elnw
    (cond
     ;; add pwd to infile if no directory in
     ;; ELIN
     ((= (length infile) 0)
      (message "usage: elnw in [out]\n")
      (kill-emacs 0))
     ((stringp infile) 
      (setq infile (expand-file-name infile))))
    ;; flag if no ELOUT
    (cond
     ((= (length outfile) 0) 
      (setq outfile nil))
     (t (setq outfile (expand-file-name infile))))    
    ;; if no ELOUT
    (cond
     ((not outfile)
      (cond
       ;; if ELIN has no file extension
       ;; assume .nw.el for it and use .nw for ELOUT
       ((not (file-name-extension infile))
	(setq outfile (concat infile ".nw"))
	(setq infile (concat infile ".nw.el")))
       ;; if ELIN has extension .nw.el use 
       ;; .el for ELOUT
       ((string-match "\\.nw\\.el$" infile)
	(setq outfile (file-name-sans-extension infile))))))
    ;; now call processing function
    (cond
     ;; if no infile just message an quit
     ((not (stringp infile))
      (message "usage: elnw infile [outfile]\n"))
     ;; if file infile doesn't exist message and quit
     ((not (file-exists-p infile))
      (message (format "cannot open: %s\n" infile)))
     (t
      (setq in-root (file-name-sans-extension infile))
      ;; if outfile still not defined
      ;; just make it infile + .nw
      (if (not outfile)
       (setq outfile (concat in-root ".nw")))
      (elnw-el-to-nw infile outfile)))))

(noweb-doc "elnw.el"
"
~subsubsection{~emph{elnw-el-to-nw}}
The function ~emph{elnw-el-to-nw} does the
real work of converting an ~extnwel file
to a ~extnw file with proper noweb syntax.

The output file can then be processed by noweave and
notangle in the usual way to generate documentation
and extract source code.
")

(defun elnw-el-to-nw (in &optional out)
"Process the noweb-like lisp file IN and dump the
results to the true noweb style file OUT.
We open file IN and look for all occurrences
of function calls to noweb-doc which have
the form (noweb-doc 'following-code-chunk '(
any lines of stuff)). All the lines of
the 2nd argument are output as an
      @ (in column 1)
      all the stuff

Into the output file (creating a noweb doc
chunk.

We also look for all function calls
of the form (noweb-chunk root ref1 ref2 ....)
If ROOT is non nil then we ouput a code chunk of the
form:
   @<<root>>=
   <ref1>
   <ref2>
   .
   .
   .

If ROOT is nil then we just output the chunk
references (this can be used to emit chunk
references into an already started code chun.

After dumping the latex stuff we start
the named code chunk as follows (which could
be a continuation of a prior chunk)
       @<<following-code-chunk>>=
Indicating the start of a program chunk.
The resulting OUT file can be processed
with noweave and notangle to generate
literate source code listings and lisp
program files. This allows us to
use emacs as normal to edit the noweb-like
files while still producing noweb style documented
code"
   (interactive "fname of file to convert:\nFoutput:")
   (let* ((in1) (out1) (cur) (next)
	  (l-out) (l-out1) (l) (chunk) (start) (end) (end1)
	  (s-start) (s-end)
	  (funcl) (val) (i) (errors 0) (tmp) (done)
	  (defs) (root) (first-chunk)
	  (chunk-started) (preamble-inserted) (preamble))
     (noweb-doc "elnw.el"
"
We first do the standard stuff
checking for validity of the input and
out files and ensuring that our input
file has the proper file extension.

If not we abort the process.
")


    ;; check that we have input file
    (cond
     ((not (file-exists-p in))
      (setq errors -1)
      (message "cannot find file %s" in))
     (t
      ;; now open in and out files and check
      ;; out file created ok
      (setq in1 (find-file in))
      ;; if no output file default
      ;; to input file less trailing extension
      ;; if the remaining extension is nw
      ;; otherwise add .nw to input file
      (cond 
       ((and (bufferp in1) 
	     (not out))
	(setq out (file-name-sans-extension
		   in))
	(let ((ext (file-name-extension out)))	    
	  (cond
	   ((not (equal ext "nw"))
	    (setq out (concat in ".nw")))))))

      (setq out1 (find-file out))
      (cond
       ((not (bufferp out1))
	(setq errors -2)
	(message "could not open output file %s" out))
       (t
	;; clear the output file
	(with-current-buffer out1
	  (erase-buffer))
	;; setup the default root code name
	;; x.nw.el goes to x.el
	(setq root (concat 
		    (file-name-sans-extension
		     (file-name-nondirectory out)) ".el"))
	;; now scan input file looking
	;; for special function calls and
	;; expand as required
	(noweb-doc "elnw.el"
"
At this point we have validated the parameters
and begin scanning the file for our noweb-doc and
noweb-chunk references. In scanning we must check
if we find such a reference that it is indeed
the first thing on a line and looks like a valid
function call and not an in-comment or in-string
match. The test below is imperfect but should
weed out all matches of noweb-doc and noweb-chunk
themselves that are in comments or strings.
")
	(with-current-buffer in1
	  ;; fontify the buffer so we can locate
	  ;; defines in code chunks
	  ;; do this explicitly only if we are
	  ;; running in batch mode
	  (cond
	   (noninteractive
	    (elnw-load-dot-emacs)))
	  (font-lock-fontify-buffer)
	  (noweb-doc t
"Explicit syntactic fontification is required
in interactive mode - to ensure that we pick up
the variable and function definitions. We do it
all the time just to be sure.")
	  (font-lock-fontify-syntactically-region
	   (point-min) (point-max))
	  (goto-char (point-min))
	  ;; save current position in input
	  ;; buffer and scan forward for a special
	  (while (< (point) (point-max))
	    (setq cur (point) done nil)
	    (setq next (elnw-find-next-special))
	    (noweb-doc "elnw.el"
"
Once we find a reference (~emph{next} contains
the position after the reference found. We go back to
the beginning of the current line (Note: this means
straddling the opening ~emph{(} of the function
call and the function name (noweb-doc or noweb-chunk)
across 2 lines must be avoided - not necessarily
a likely occurrence except for extremely sloppy
programmers). We then scan forward to the end of the
expression (which should place us immediately after
the closing bracket of the function call. We mark the
end as our next starting point. We then go to the
beginning of the expression.
")

	    (cond
	     (next
	      (setq s-end next)
 	      (backward-sexp)
	      (setq s-start (point))
	      (noweb-doc t
"If we are at the first chunk then
we want to save any preamble code that
precedes this first chunk and insert it
into the noweb file - at the beginning of
the first code chunk.")
	      (if (not first-chunk)
		  (setq first-chunk s-start
			cur s-start
			preamble 
			(if (> s-start (point-min))
			 (buffer-substring (point-min)
					   s-start) nil)
			preamble-inserted (not preamble)))))
	    (noweb-doc "elnw.el"
"
If our current point (the start of the next
noweb special instruction) is greater than the
last point we arrived at (~emph{cur}) we want
to dump all the code as is directly to the
output buffer. Nothing needs to be done with
this stuff. So we just dump it in.
")

	    ;; if we found a forward special
	    ;; then write out all stuff between
	    ;; previous point and start of the special
	    (cond 
	     ((> (point) cur)
	      (save-excursion
		(save-restriction
		  (narrow-to-region cur (point))
		  (setq defs (elnw-find-definitions))
		  (setq l-out (buffer-string))))
	      (save-excursion
		(with-current-buffer out1
		  (goto-char (point-max))
		  (insert l-out)
		  (insert "\n")))))
	    (noweb-doc "elnw.el"
"
Now if ~emph{next} is nil it means we did not find
any more specials. So we set ~emph{l-out}, the string
we will dump, to the remainder of the buffer, from
our last point ~emph{cur} to the end of the file.
This will be copied as is.

Otherwise we did find a special so we start examining the
arguments. We move forward 1 character(inside the opening
bracket) and gather all the arguments (including the function
name into the list ~emph{funcl}. This is fairly
straightforward - except I decided to allow quoted
arguments in the list of arguments. So any quoted
arguments are stripped of the leading quote. Inside
quoted arguments we may have an opening bracket or
an opening string quotation. If we find these we take
just everything inside the surrounding brackets or
quotation markes as the total argument.

For the standard case of pure string arguments - the
entire string is the sexp and we use it as is.
")

	    (cond
	     ((not next)
	      (setq l-out (buffer-substring cur (point-max)))
	      (goto-char (point-max)))
	     (t
	      (forward-char 1)
	      (setq funcl nil)
	      (while (< (point) next)
		(condition-case nil
		 (progn
		   (forward-sexp)
		   (setq end (point))
		   (backward-sexp)
		   (setq start (point))
		   ;;(setq val (thing-at-point 'sexp))
		   (setq val (buffer-substring start end))
		   (if (string= (substring val 0 1) "'")
		     (setq val (substring val 1)))
		   (cond
		    ((or (setq i (string-match "\\\"" val))
			 (setq i (string-match "(" val)))      
		     (if (= i 0)
		      (setq val (substring val 1 (- (length val) 1))))))
		   (setq funcl (append funcl (list val)))
		   (forward-sexp))
		 (error
		  (goto-char next))))
	      (noweb-doc "elnw.el"
"
Now that ~emph{funcl} contains the total argument list
we can see what we have and what we need to do next.

If we a noweb-doc call that has exactly 3 arguments
then we can proceed with it.

First we take the doc string the 2nd argument 
of the function call and replace the user's 
chosen latex macro start marker with a real
backslash everywhere found.

We then dump the noweb @ marker to the
output (flagging start of a doc chunk) followed
by the actual translated text, completing the documentation
chunk.

If argument 1 was not nil - then we use it as a 
root code chunk name and open a new (or continuation
code chunk in the standard noweb format.

If it is a noweb-chunk function call we simply
dump the opening code chunk reference and then 
drop in all the arguments as chunk references.
")

	      (goto-char next)
	      (setq chunk-started nil
		    l-out1 nil)
	      (cond
	       ((and (string= (car funcl) NOWEB-DOC)
		     (= (length funcl) 3))
		(setq tmp
		      (elnw-replace-latex-macro-marker 
		       (nth 2 funcl)))
		(setq l-out 
		      (concat 
		       "@ "
		       (if defs 
			   (concat "%def "
				   (mapconcat 'concat defs " ")) "")
		       "\n"
		       tmp))
		(cond 
		 ((and (not (string= "nil" (nth 1 funcl)))
		       (not (string= "t" (nth 1 funcl))))
		  (setq root (nth 1 funcl))))
		(cond
		 ((not (string= (nth 1 funcl) "nil"))
		  (setq chunk-started t)
		  ;; if no explicit name for first
		  ;; chunk (root nil) then set root
		  ;; to the extensionless input file
		  ;; appended with .el
		  (cond
		   ((not root)
		    (setq tmp (file-name-nondirectory in))
		    (while (file-name-extension in)
		      (setq tmp (file-name-sans-extension tmp)))
		    (setq root (concat tmp ".el"))))		    
		  (setq l-out 
			(concat 
			 l-out 
			 (format "\n%s%s>>=\n" 
				 ELNW-ANG-OPEN root))))))
	       ;; this is a sam-doc-chunk-ref function
	       ;; just insert all the references between
	       ;; double-angle brackets in current chunk
	       ((and (string= (car funcl) NOWEB-CHUNK)
		     (> (length funcl) 1))
		(setq l-out "\n")
		(cond
		 ((not (string= (nth 1 funcl) "nil"))
		  (setq chunk-started t)
		  (setq l-out
			(format "\n%s%s>>=\n" 
				ELNW-ANG-OPEN (nth 1 funcl)))))
		(mapcar 
		 (lambda (x)
		   (setq l-out1 (concat l-out1 
					ELNW-ANG-OPEN x ">>\n")))
		 (cddr funcl)))
	       ;; this is a non lisp code chunk
	       ((and (string= (car funcl) NOWEB-CODE)
		     (= (length funcl) 3))
		(setq l-out 
		      (concat 
		       (format "\n%s%s>>=\n" 
			       ELNW-ANG-OPEN (nth 1 funcl))
		       (nth 2 funcl)
		       "\n")))
	       (t
		(setq errors (1+ errors))
		(setq l-out 
		      (concat
		       "***syntax-error - bad noweb call\n"
		       (buffer-substring s-start s-end)))))))
	    (noweb-doc "elnw.el"
"
At this point ~emph{l-out} contains the string
replacement for our noweb-doc and noweb-chunk
function calls so we drop the result into the 
outout buffer. We then go back and look for
the next special form to replace.

Once we are done we save the output buffer and 
close the output file.

The function returns the number of
syntax errors found. We need a better method
of flagging errors in conversion. This should be
fixed.
")


	    (cond 
	     ((stringp l-out)
	      (save-excursion
		(with-current-buffer out1
		 (goto-char (point-max))
		 (insert l-out)
		 (insert "\n")
		 (noweb-doc t
"If we have a preamble (lisp code before the
first noweb-doc or chunk call) and we
are at the beginning of the first code-chunk
then we insert our preamble now, and mark it
as inserted.")
		 (cond
		  ((and (not preamble-inserted)
			first-chunk chunk-started)
		   (setq preamble-inserted t)
		   (insert preamble)))
		 (noweb-doc t
"l-out1 contains any additional stuff to go into
a chunk after a potential preamble. If we have some stuff
there then insert it now.")
		 (if l-out1 (insert l-out1)))))))
	  (set-buffer-modified-p nil)
	  (kill-buffer nil)
	  (with-current-buffer out1
	    (basic-save-buffer)
	    (kill-buffer nil)))))))
    (if (> errors 0)
      (message (format "Syntax Errors Found (%d)" errors)))
    errors))


(noweb-doc "elnw.el"
"
~subsubsection{~emph{elnw-compile}}
The function elnw-compile is also a function
with no arguments, intended for use from a shell-script
to initiate elnw file conversion (if necessary) and
extraction and compilation of all root files.

Again it takes the input filename from environment
variable ELIN. The output filename is inferred.
")

(defun elnw-compile ()
  (let ((infile (getenv "ELIN")))
    (cond
     ((and (stringp infile) 
	   (> (length infile) 0))
      (setq infile (expand-file-name infile))))
    (elnw-byte-compile-file-all infile)))

(noweb-doc "elnw.el"
"
~subsubsection{Byte Compile all .el Roots in File/Buffer}

The following 2 functions process all the root
lisp files in the input file or input buffer
which may reference  either ~extnwel or ~extnw files. 
The ~extnwel file is converted to pure
noweb form first if required. All the roots are
extracted with notangle and byte compiled with
the total process logged in the compilation
log.
")

(defun elnw-byte-compile-file-all (fn)
"Extract and compile all el files from noweb file FN"
  (interactive "ffile name:")
  (let ((fn1 (elnw-check-file fn))
	(roots))
   (cond
    ((and fn1 (> (length fn1) 0))
     (setq roots (elnw-get-roots fn1))
     (mapcar (lambda (x)
	       (let ((ext (file-name-extension x)))
		 (cond
		  ((equal ext "el")
		   (elnw-byte-compile-file-root fn1 x))
		  (t
		   (elnw-log
		    (format "skipping non el root:%s\n" x))))))
	     roots))
    (t
     (elnw-log 
      (format "Input noweb file not found:%s\n" fn))))))
    

(defun elnw-byte-compile-buffer-all ()
"Byte compile the all the .el root files extracted from  
current nw buffer"
  (let ((fn (buffer-file-name))
	(ans))
    (cond
     ((string-match "\\(\.nw$\\)\\|\\(\.nw\.el$\\)" fn)
      (cond
       ((buffer-modified-p)
	(setq ans (read-string "save buffer[y]:"))
	(cond
	 ((or (= (length ans) 0)
	      (string-match "^[yY]" ans))
	  (basic-save-buffer)))))
      (elnw-byte-compile-file-all (buffer-file-name)))
     (t
      (message "not a .nw or .nw.el file\n")))))

(noweb-doc "elnw.el"
"
~subsubsection{~emph{Compile Specific Root in Buffer or File}}
The following functions can be invoked
in emacs mode to perform extraction and
byte-compilation of lisp source code
from a noweb-elisp file in one step.

The function ~emph{elnw-byte-compile-file-root}
takes a filename and a root name argument. The root
name specified should refer to a target ~extel file.
If no ~extel extension is in the root name one will
be appended. If the input file needs conversion to
noweb format (of file extension  ~extnwel) the
conversion will be done. The specified root file
will be extracted with notangle and the resulting
file byte-compiled.

The results of the above processing are logged into
the *Compile-Log* buffer.
")

(defun elnw-byte-compile-file-root (fn root)
"Take the file associated with the  file FN
 - which must have extension .nw.el or .nw. 
If extension is .nw.el then using elnw first
prepare the .nw (noweb) file. 

Then invoke notangle to extract the root ROOT
 .el sources.
Then invoke byte-compile-file to compile the
extracted ROOT.

If ROOT has no .el extension one is attached
and it must match a root of the same name in the
noweb doc/source file."
  (interactive "finput file:\nsroot name:")
  (let ((fn1 (elnw-check-file fn))
	(roots) (out))
    (cond 
     ((and fn1 (file-exists-p fn1))
       (setq roots (elnw-get-roots fn1))
       (if (not (string-match ".el$" root))
	   (setq root (concat root ".el")))
       (cond
	((member root roots)
	 (setq out (concat (file-name-directory fn1) root))
	 (if (file-exists-p out)
	  (delete-file out))
	 (shell-command-to-string
	  (format "%s -R%s %s >%s"
		  ELNW-NOTANGLE root fn1 out))
	 (cond 
	  ((file-exists-p out)
	   (find-file out)
	   (goto-char (point-min))
	   (while (not (eobp))
	     (cond
	      ((= (length (split-string (thing-at-point 'line))) 0)
	       (kill-line))
	      (forward-line 1)))
	   (basic-save-buffer)
	   (kill-buffer nil)
	   (elnw-log (format "\nExtracted file:%s\n" out))
	   (if (byte-compile-file out)
	    (elnw-log 
	     (format "\nSuccessfully compiled file:%s\n" out))))))
	(t
	 (elnw-log 
	  (format "Error: %s has no root chunk %s" fn root))))))))

(noweb-doc "elnw.el"
"
The function ~emph{elnw-byte-compile-buffer-root}
may be invoked in emacs mode directly or as an interactive
alt-x function. It takes 1 argument, a 
target root. The buffer should reference a ~extnwel or
~extnw file. If necessary the buffer file will
be converted to pure noweb (~extnw) format first.
The named root (referencing an ~extel lisp source 
file will be extracted with notangle and the source file
will be byte-compiled.
")

(defun elnw-byte-compile-buffer-root (root)
"Byte compile the .el file ROOT extracted from  
current nw buffer."
  (interactive "sroot name:")
  (let ((fn (buffer-file-name))
	(ans))
    (cond
     ((string-match "\\(\.nw$\\)\\|\\(\.nw\.el$\\)" fn)
      (cond
       ((buffer-modified-p)
	(setq ans (read-string "save buffer[y]:"))
	(cond
	 ((or (= (length ans) 0)
	      (string-match "^[yY]" ans))
	  (basic-save-buffer)))))
      (elnw-byte-compile-file-root (buffer-file-name) root)))))

(noweb-doc "elnw.el"
"
~subsubsection{~emph{elnw-gen-docs}}
The function elnw-gen-docs is also a function
with no arguments intended as the entry point
for a shell script initiating emacs in batch mode
to perform noweb document generation.

It takes the arguments from environment variables
ELIN and ELOUT set by the shell script. ELIN
specifies the input filename (typically a 
~extnwel file. This file will be first 
transformed to a pure ~extnw file with elnw
conversion and then noweave will be run followed
by latex and pdflatex to generate the documentation
for the file.

The ELOUT environment variable, if specified,
is the target directory for the documentation files.
If the directory is a relative pathname then the
docs will go into the directory relative to
the directory of the source file. If it not
specified then the docs go into the directory
of the source file. Otherwise they go into the
absolute path specified.

After check the arguments for correctness this
function simply calls elnw-generate-docs to do
the real work.
")


(defun elnw-gen-docs (&optional in out)
  (let ((infile (getenv "ELIN"))
	(dir (getenv "ELOUT")))
    (cond
     ((and (stringp infile) 
	   (> (length infile) 0))
      (setq infile (expand-file-name infile)))
     (t
      (message "usage: elnw-gen-docs infile [dir]")
      (kill-emacs 1)))      
    (cond
     ((and (stringp dir)
	   (> (length dir) 0))
      (setq dir (expand-file-name (concat dir "/"))))
     (t (setq dir nil)))
    (elnw-generate-docs infile dir)))

(noweb-doc "elnw.el"
"
~subsubsection{~emph{elnw-generate-docs}} 
The function ~emph{elnw-generate-docs} takes
2 arguments an input file (which may be either
pure noweb format - extension ~extnw , or elisp
noweb format - extension ~extnwel), and an 
optional output directory.

It will process the (possibly converted by
~emph{elnw-el-to-nw} function) noweb
file with noweave and use the elnw-fontify-filter to
present colorized pdf and dvi documentation.

If the output directory (2nd argument) is provided
it specifies either an absolute directory path or
a path relative to the input file directory,
in which to store the documentation.

In interactive mode, the system will first re-create
all the face-color mapping and pass it to the
colorization filter. The colorization filter
looks for a file .~,noweb-font-lock-colors to
pick up the font-lock face colors. This file
will be generated, if necessary (i.e. not available).

The output tex file from noweave is passed through
the elnw-fontify-filter function with the -delay option 
 (which excludes the standard noweb latex preamble.

This function edits the resultant tex output and
inserts an appropriate preamble including the
xcolor package definition, as well as appending the
trailing latex end document.

The file is then passed through latex and pdflatex.

The entire process is logged either to the compile-log buffer
in interactive mode or to the terminal in batch mode.
")

(defun elnw-generate-docs (in out-dir)
"Generate latex and pdf docs from input
file FN. If FN has extension NW then the docs are
generated directly. If FN has extension NW.EL then
we first transform the file to NW (noweb 
format using eellnw above. Then using that NW file
we generate all the doc files."
  (interactive "fInput file:\nDOutput Directory:")
  (noweb-doc "elnw.el"
"Get the in and out-dir from environment
variables if necessary. Check the input
file and convert to noweb format if required
and set the out-dir as either absolute or relative
if provided or to the directory of in file if not
provided.")
  (let ((fn1 (elnw-check-file in))
	(dir (cond
	      ((not out-dir) 
	       (file-name-directory in))
	      ((file-name-directory out-dir)
	       (file-name-as-directory out-dir))
	      (t
	       (concat (file-name-directory in) out-dir))))
	(out) (base))
    (cond
     (fn1
      (kill-buffer nil)
      (setq base (file-name-nondirectory
		  (file-name-sans-extension fn1)))
      (setq out (concat dir base))
      (noweb-doc "elnw.el"
"If we are in interactive mode then dump the
font-lock face colors to ~/.noweb-font-lock-colors
for the elnw-filter colorization.

Once this is done invoke noweave on the input
file and generate the colorized latex output.

Call elnw-fixup-tex to insert the preamble and
end doc in the tex file then process it with
latex (twice - to create the toc if needed) and
then pdflatex to generate the pdf version of the
document.
")
      (cond
       ((not noninteractive)
	(elnw-make-colormap)))
      (elnw-log
       (shell-command-to-string
	(concat 
	 ELNW-NOWEAVE
	 " -index -n -filter elnw-filter " fn1 " >"
	 out ".tex")))
      (noweb-doc t
"Here we call ~emph{elnw-fixup-tex} to
insert the preamble and post-fix to the
raw tex file suitable for making colored xdvik
dvi output.")
      (elnw-fixup-tex (concat out ".tex") t)
      (elnw-log
       (shell-command-to-string
	(concat "cd " dir ";" ELNW-LATEX " " 
		(concat base ".tex"))))
      (elnw-log
       (shell-command-to-string
	(concat "cd " dir ";" ELNW-LATEX " " 
		(concat base ".tex"))))
      (noweb-doc t
"Now we re-fix up the tex document suitable for
colored pdflatex output to a pdf file.")
      (elnw-fixup-tex (concat out ".tex") nil)
      (elnw-log
       (shell-command-to-string
	(concat "cd " dir ";" ELNW-PDFLATEX " "
		(concat base ".tex"))))
      (elnw-log
       (shell-command-to-string
	(concat "cd " dir ";" ELNW-PDFLATEX " "
		(concat base ".tex"))))))))



(noweb-doc "elnw.el"
"
~subsubsection{~emph{elnw-fontify}}
The function elnw-fontify is the main entry
point for applying font lock to a file
from a filter function and transforming it
into a ~LaTeX encoded colorized version.

If the optional IN and OUT input, output
filenames are nil it goes to environment
variables ELIN and ELOUT (so that this
function too may be called from a shell
script if needed).

It first does some standard checking and setup
to ensure validity and consistency of the
input and output filenames either passed 
directly or garnered from the environment variables.

If all is okay e then proceed to the actual
translation to a ~LaTeX colorized version of
the source code in the input file.

The extension of the input file indicates
how the file is to be fontified, automagically
by emacs. As long as the user's standard 
environment loads and fontifies files of the
specified extension properly - this fontifier
will do so as well - from batch mode - since
it uses the user's own .~,emacs file to accomplish
the task. 
")

(defun elnw-fontify (&optional in out preamble)
"Accept the file ELIN. It should be a source file
of some kind (assume now it is .el). Load it into
a buffer and fontify it with font-lock mode.
Then output a colorized version of the file to
either ELOUT if it is specified or to stdout."
  (let* ((infile (if in in (getenv "ELIN")))
	 (outfile (if out out (getenv "ELOUT")))
	 (latex-preamble (if preamble preamble
			   (getenv "PREAMBLE")))
	 (home) (ext)
	 (wd (nth 1 (split-string (pwd))))
	 (inb) (outb) (cur) (next-change) (fcolor)
	 (nextcolor))
    
    (cond
     ((and (stringp infile) 
	   (> (length infile) 0))
      (setq infile (expand-file-name infile))))

    (cond
     ;; add pwd to outfile if no directory in
     ;; ELOUT
     ((and (stringp outfile) 
	   (> (length outfile) 0))
      (setq outfile (expand-file-name outfile))))
    
    (cond
     ((not (file-exists-p infile))
      (message "no input file specified\n")
      (setq infile nil))
     (t
      (setq ext (file-name-extension infile))
      (setq inb (find-file infile))))
    
    (setq outb (find-file outfile))
    (cond
     ((or (not (bufferp inb))
	  (not (bufferp outb)))
      (message "failed to open output file\n")
      (setq infile nil)))
    
    (with-current-buffer outb
      (erase-buffer)
      (if latex-preamble
	(insert ELNW-TEX-RAWCODE-PREAMBLE-XDVI)))

    (noweb-doc "elnw.el"
"
We are now ready to go. All parameters have
been validated, and the input and output files
checked and opened.

We first load the user's .~,emacs file
to setup with user specials as needed.
")


    (elnw-load-dot-emacs)

    (noweb-doc "elnw.el"
"
We first invoke ~emph{font-lock-fontify-buffer}
to fontify and colorize the entire buffer.

Once this is done we start scanning the
buffer from the beginning looking for
changes in the text face-color.
Function ~emph{elnw-get-face-color} returns
the color at the current point in the buffer.
")

    (cond
     (inb
      (save-excursion
	(with-current-buffer inb
	  (require 'font-lock)
	  (font-lock-fontify-buffer)
	  (goto-char (point-min))
	  (while (not (eobp))
	    (setq cur (point) 
		  next-change cur
		  fcolor (elnw-get-face-color)
		  nextcolor fcolor)
	    ;; check for any definitions at point
	    (while (and (not (eobp))
			(equal fcolor nextcolor))
	      (setq next-change 
		    (or (next-property-change next-change
					      (current-buffer))
			(point-max)))
	      (goto-char next-change)
	      (setq nextcolor (elnw-get-face-color)))
	    (noweb-doc "elnw.el"
"
At this point we are positioned at ~emph{next-change}
and ~emph{cur} holds the position we moved from. The
entire region between these points is colorized
with color ~emph{fcolor} and ~emph{nextcolor}
holds the new color from the current point onwards.

Since we don't want newlines mapped inside
our textcolor ~LaTeX macro calls (this screws
up our line breaks) we must split the text in
this region into separate lines, colorize
each line individually separated by the line breaks.

The function ~emph{colorize} is invoked to take
each substring and translate it into a textcolor
macro call and dump the result to the output 
buffer specifed by ~emph{outb}.

Finally when we are done with the region we set 
~emph{fcolor} to ~emph{nextcolor} and go back and
look for the next region of similarly colored
text.

This repeats until we are done.

Once we have finished the translation we
clean up the buffers, saving the ouput and
terminate.
")
	    
	    (cond
	     ((> next-change cur)
	      (let ((lines (buffer-substring cur next-change))
		    (i))
		(while (and lines (setq i (string-match "\n" lines)))
		  (if (> i 0)
		      (elnw-colorize outb (substring lines 0 i) fcolor))
		  (elnw-writeb outb "\n")
		  (if (< (length lines) 2)
		      (setq lines nil)
		    (setq lines (substring lines (1+ i)))))
		(if (> (length lines) 0)
		    (elnw-colorize outb lines fcolor)))))
	    (setq fcolor nextcolor))))
      (with-current-buffer outb
	(goto-char (point-max))
	(if latex-preamble
	   (insert ELNW-TEX-RAWCODE-FINAL))
	(basic-save-buffer))
      (kill-buffer inb)
      (kill-buffer outb)))))
	  
(noweb-doc "elnw.el"
"
~subsubsection{~emph{elnw-fontify-filter}}

The ~emph{elnw-fontify-filter} takes either 
INF (input file) or if INF is nil reads from stdin
and writes output to either OUTF (output file) or
if nil sends output to stdout.

Hence it serves as a proper filter for noweave.

Its purpose is to pass through everything except 
code chunks. Code chunks are saved to a temp
file. When complete the file is passed to routine
~emph{elnw-fontify} for font-lock and colorization
of code chunks which are latex encoded and then
dumped to the output stream again by this filter.
")

(defun elnw-fontify-filter (&optional inf outf)
"This function is mostly called from batch mode
without any arguments.
It behaves as a noweb filter reading stdin
and writing to stdout.

All documentation chunks are passed through as is.
Code chunks are buffered to a temporary file.
When the end of each code chunk is received,
it is fontified according to the current
language (as specified in @language directives
in the input) and the fontified source
code (colorized) is passed out. It calls 
the preceding function"
  (let ((use-list) (lang) (comment-marker) (l)
	(code) (begin-code) (end-code)
	(tmpb) (outn) (fn) (inb) (outb)
	(lang-entry) (code-stuff)
	(flm (concat (file-name-as-directory (getenv "HOME"))
		     ELNW-FONT-LOCK-MAP)))

    (noweb-doc "elnw.el"
"We first see if we had string arguments. If
so we set up our input and output buffer 
pointers. If ~emph{inb} or ~emph{outb} 
 (the buffer pointers) are nil then the 
filter read and write functions ~emph{elnw-read-one-line}
and ~emph{elnw-write-one-line} will use stdin and stdout
as the i/o rather than the files.

We then load the users ELNW-FONT-LOCK-MAP file if we 
can find one - otherwise we will use the built-in
defaults.
")
    (cond
     ((stringp inf)
      (setq inb (find-file inf))
      (goto-char (point-min))))
    (cond
     ((stringp outf)
      (setq outb (find-file outf))
      (with-current-buffer outb
	(erase-buffer))))

    (cond
     ((file-exists-p flm)
      (load-file flm)
      (condition-case nil
	  (setq elnw-default-face-colors (symbol-value 'noweb-font-lock-colors))
	(error
	 nil))))
		      
    (noweb-doc "elnw.el"
" We now start reading from the input source one
line at a time. The input is in noweb markup format.
We look for code chunks delimited by @begin code num
and save all lines until we hit an @end code marker.
This flags a single chunk that needs to be latex 
colorized and fontified. All other lines we pass through.

We trap out @language lang-name statements which flag
the programming language for the next chunk. This will be
used as a file extension so that emacs will fontify
properly according the the users .~,emacs file settings.

We also trap out any lines of the form @comment-marker chars.
For languages not known internally, the comment marker
tells us what is the necessary single line comment characters
to use (e.g. ;; in elisp or // in c or ~# in tex etc.).

We need such markers because we pass any @use statements
directly through as comments and uncomment them on the way out.

The @use statements are references to used chunks.
")    
    (while (setq l (elnw-read-one-line inb))
      (cond
       ;; trap out lines anywhere in doc or code 
       ;; chunks which are of form @text @language lang
       ;; and save current language
       ((string-match "^@text @language \\(.*\\)" l)
	(setq lang (match-string 1 l)))
       ((string-match "^@text @comment-marker \\(.*\\)" l)
	(setq comment-marker (match-string 1 l)))
       ;; not an @begin code line
       ;; just pass it through the filter
       ((not (string-match "^@begin code \\(.*\\)" l))
	(elnw-write-one-line outb (format "%s\n" l)))
       ;; we got an @begin code marker 
       (t
	(noweb-doc "elnw.el"
"We now are starting a code chunk. 
Save the begin-code line in the list
code-stuff and then keep reading
until we hit the @end code line.
")
	(setq code-stuff (list l))
	(while (and (setq l (elnw-read-one-line inb))
		    (not (string-match "^@end" l)))
	  (setq code-stuff (append code-stuff (list l))))
	(setq end-code l)
	(setq fn (make-temp-file "/tmp/filter"))
	(setq tmpb (find-file fn))
	(noweb-doc "elnw.el"
"Now process all the buffered lines.
Trap out the @text @language or @comment-marker lines.
For all others extract the text,nl (newline) or use, lines
and feed them into the temp file. Anything else we pass through
immediately.
")
	(while code-stuff
	  (setq l (pop code-stuff))
	  (cond
	   ((string-match "^@text @language \\(.*\\)" l)
	    (setq lang (string-match 1 l)))
	   ((string-match "^@text @comment-marker \\(.*\\)" l)
	    (setq comment-marker (match-string 1 l)))
	   ((string-match "^@text \\(.*\\)" l)
	    (elnw-writeb tmpb (match-string 1 l)))
	   ((string-match "^@nl$" l)
	    (elnw-writeb tmpb "\n" ))
	   ((string-match "^@use" l)
	    (elnw-writeb tmpb 
		    (concat 
		     (elnw-get-comment-marker comment-marker lang) 
		     l)))
	   (t
	    (elnw-write-one-line outb (concat l "\n")))))
	(noweb-doc "elnw.el"
"Now that we have the stuff in the temp file
we want to fontify it. 

We create a temporary output filename for 
the fontified result. Then we need to find
a file extension for the current language, 
and rename the file  if necessary, with the new 
extension, so that font-lock will do its
magic properly. For shell scrips and other
stuff font-lock magic still works with no extensions.
")
	(setq outn (make-temp-file "/tmp/filter"))
	(with-current-buffer tmpb
	  (basic-save-buffer))
	(kill-buffer tmpb)
	(setq lang-entry (assoc lang elnw-lang-flags))
	(cond
	 (lang-entry
	  (setq tmpb (concat fn "." (nth 1 lang-entry)))
	  (rename-file fn tmpb)
	  (setq fn tmpb)))
	(noweb-doc "elnw.el"
"We now pass to the main fontifier.
When done we should have the result in
file ~emph{outn} all nicely LaTeX encoded.

We open the output file and
start reading it a line at a time.
We undo whatever we did for the @use reference
lines. All other lines we pass back
as @literals to the filter output.

When done with the intermediate files
we get rid of them.
")
	;; now pass file for fontification
	(elnw-fontify fn outn)
	(setq tmpb (find-file outn))
	(cond
	 ((bufferp tmpb)
	  (with-current-buffer tmpb
	    (goto-char (point-min))
	    (while (not (eobp))
	      (setq l (thing-at-point 'line))
	      (forward-line)
	      (cond
	       ((string-match 
		 (concat
		  (elnw-get-comment-marker comment-marker lang)
		  ".*\\(@use [^{}]*\\)") l)
		(setq l (match-string 1 l))
		(elnw-write-one-line outb (concat l "\n"))
		(elnw-write-one-line outb (concat "@text \n@nl\n")))
	       (t
		(elnw-write-one-line outb (format "@literal %s" l))
		(elnw-write-one-line outb "@nl\n"))))
	    (elnw-write-one-line outb (format "%s\n" end-code)))
	  (kill-buffer tmpb)
	  (delete-file outn)))
	(delete-file fn))))
    (noweb-doc "elnw.el"
"
Now we just see if we were reading/writing to
a file and if so we close the input file and
save the output file, and kill both buffers.
")
    (cond
     (inb
      (kill-buffer inb)))
    (cond
     (outb
      (with-current-buffer outb
	(basic-save-buffer)
	(kill-buffer outb))))))
      
(noweb-doc "elnw.el"
"subsubsection{~emph{elnw-get-roots}}
The function ~emph{elnw-get-roots} 
invokes the noweb utility ~emph{noroots}
to extract all the noweb roots from a noweb
lisp or pure noweb file. If no filename
is supplied it defaults to the file-name of
the current buffer.
")

(defun elnw-get-roots (&optional fn)
  (interactive)
  (let ((fn1)
	(roots))
   (cond
    ((not fn)
     (setq fn (buffer-file-name))))
   (setq fn1 (elnw-check-file fn t))
   (cond
    ((stringp fn1)
     (setq roots 
	   (split-string 
	    (shell-command-to-string
	     (concat ELNW-NOROOTS " " fn1)) "\n"))
     (setq roots
	   (mapcar (lambda (x)
		     (let ((a))
		       (cond
			((string-match 
			  (concat ELNW-ANG-OPEN "\\(.*\\)>>") x)
			 (setq a (match-string 1 x))
			 (if (> (length a) 0) a nil))
			(t nil))))
		   roots))
     (setq roots (delq nil roots))))
   roots))

(noweb-doc "elnw.el"
"
~subsubsection{Hide/Show noweb-docs}
The following interactive functions allow
user to hide or show all or individual
noweb-doc, noweb-chun or noweb-other-code
calls.")

(defun elnw-hide-all ()
"Hide all noweb doc stuff in current buffer"
   (interactive)
   (elnw-hide-show-all t))

(defun elnw-show-all ()
"Show all noweb doc stuff in current buffer"
   (interactive)
   (elnw-hide-show-all nil))

(defun elnw-hide ()
  (interactive)
  (elnw-hide-show-all t (point)))

(defun elnw-show ()
  (interactive)
  (elnw-hide-show-all nil (point)))

(noweb-doc t
"subsubsection{~emph{elnw-make-colormap}}
This function must be executed in interactive
mode. It takes the user font-lock color map
and saves it in a lisp file called 
HOME/.noweb-font-lock-colors. This map is
used during filter-mode fontification to
pick up the user preferred color preferences.
The preferences need to be resident here, since
during a filter level fontification, the
color map is not available since emacs will be running
in batch mode and font-lock associations are only
established when a window frame has been created.")

(defun elnw-make-colormap ()
"Create the file ~/.noweb-font-lock-colors 
if we are in interactive mode and the file
does not already exist."
   (interactive)
   (let ((flm (concat (file-name-as-directory (getenv "HOME"))
		      ELNW-FONT-LOCK-MAP))
	 (flb) (faces))
     (cond
      ((not (file-exists-p flm))
       (setq faces (delq nil
			 (mapcar 
			  (lambda (x)
			    (if (string-match "font-lock-" 
					      (symbol-name x))
				(list x (face-foreground x))))
			  (face-list))))
       (save-excursion
	 (setq flb (find-file flm))
	 (cond 
	  ((bufferp flb)
	   (erase-buffer)
	   (insert 
	    (prin1-to-string (list 'defvar 'noweb-font-lock-colors 
				   (list 'quote faces))))
	   (basic-save-buffer)
	   (kill-buffer nil))))))))

(noweb-doc "elnw.el"
"
~subsection{Internal Functions}
The functions in this section are used
internally and are not intended for 
external use.
~subsubsection{~emph{elnw-find-next-special}}

Starting from point find the next noweb call
 (any of them) and return the point just after
the end of the function call s-exp.
")

(defun elnw-find-next-special ()
"Move to the beginning of the next noweb-doc,
noweb-chunk or noweb-other-code call in the 
current buffer. Moves to point just after
noweb call marker and returns point at end
off call or nil if no such calls found."
   (let ((special 
	  (concat "\\((" 
		  NOWEB-DOC   " \\)\\|\\(("
		  NOWEB-CHUNK " \\)\\|\\(("
		  NOWEB-CODE  " \\)"))
	 (done) (n1))
     (while (not done)
       (setq n1
	     (re-search-forward special nil t))
       (cond
	(n1
	 (beginning-of-line)
	 (forward-sexp)
	 (cond
	  ((equal (char-before) ?\))
	   (setq n1 (point))
	   (setq done t))
	  (t (goto-char n1))))
	(t (setq done t))))
     n1))

(noweb-doc "elnw.el"
"
~subsubsection{~emph{elnw-hide-show-all}}
Hide or show all noweb doc calls, or only the
call containing point LOC. Uses the
hideshow package to perform the invisible
overlay creation/destruction. This supports
all the interactive calls for hide/show of noweb doc
calls."
)

(defun elnw-hide-show-all (hide &optional loc)
"Hide/Show noweb doc calls. If LOC specified
then do it only for noweb-doc call which includes
point LOC"
 (let ((next) (done) (start) (end)
       (rstart)
       (flag (if hide 'code nil))
       (version (string-to-number
		 (car (split-string emacs-version "\\.")))))
  (save-excursion
    (goto-char (point-min))
    (while (not done)
      (setq next (elnw-find-next-special))
      (cond
       (next
	(setq end (point))
	(backward-sexp)
	(setq rstart (point))
	(end-of-line)
	(setq start (point))
	(cond
	 ((or (not loc)
	      (and (>= loc rstart)
		   (< loc end)))
	  (goto-char rstart)
	  (cond
	   (hide
	    (hs-hide-block))
	   (t
	    (hs-show-block)))))
	(goto-char next))
       (t (setq done t)))))))
	


(noweb-doc "elnw.el"
"
~subsubsection{~emph{elnw-load-dot-emacs}}
The function ~emph{elnw-load-dot-emacs} simply
finds and loads the .~,emacs file of the user
or dumps an error message if no file was
found.
")

(defun elnw-load-dot-emacs ()

  (let ((home (file-name-as-directory (getenv "HOME"))))
    (cond 
     (elnw-dot-emacs-loaded nil)
     ((not (file-exists-p (concat home ".emacs")))
      (message "cannot find ~/.emacs for user face rules."))
     (t
      (load (concat home ".emacs") t nil t)
      (setq elnw-dot-emacs-loaded t)))))

(noweb-doc "elnw.el"
"
~subsubsection{~emph{Color Functions}}
The function ~emph{elnw-colorize} is called to colorize
a string and dump it to an output buffer.

It invokes ~emph{elnw-color-start} to dump the macro
header and ~emph{elnw-color-end} to dump the macro
close. In the middle we have to go through
the messiness of translating and escaping 
any ~LaTeX special characters found in the
string so as not to mess up ~LaTeX itself
when it must go about the business of generating
the actual pretty documents.

elnw-colorize simply adds a latex textcolor macro
call with a macro header, then 
fixes up the text, escaping any latex special
characters found there, and finally closes
out the macro call.
")


(defun elnw-colorize (buf stuff color)
  (cond
   (color
    (elnw-color-start buf color)))
  (elnw-writeb buf (elnw-latex-escape-specials stuff))
  (cond
   (color
    (elnw-color-end buf))))

(defun elnw-color-start (outb color)
  (elnw-writeb outb
	       (concat "\\textcolor{" color "}" "{")))

(defun elnw-color-end (outb)
  (save-excursion
    (with-current-buffer outb
      (insert "}"))))

(noweb-doc t
"~subsubsection{elnw-get-face-color}
elnw-get-face-color returns the string representing
the color of the face at the current point in the buffer
 (nil implies no color). It does this by 
retrieving the face symbol and then looking up the
color in the elnw-default-face-colors association table.
This table will either use the builtin default table, or
more usually it will reference the users own table, saved
in HOME/.noweb-font-lock-colors which overrides the 
builtin defaults. Executing elnw-make-colormap in
interactive mode creates this file.
")

(defun elnw-get-face-color ()
  (let ((color) (face))
    (setq face (get-text-property (point) 'face)
	  color (nth 1 (assoc face elnw-default-face-colors))))) 
		    

(noweb-doc "elnw.el"
"
~subsubsection{elnw-latex-escape-specials}
The function ~emph{elnw-latex-escape-specials} has the
job of translating the input string so as to escape
all the ~LaTeX special characters.

It uses a temporary buffer and inserts the
string into it, then calls ~emph{latex-escape}
to do the actual work.

It first checks for all escaped backslahes
and fixes them up with a marker (a math mode
backslash marker).

It then goes through the buffer escaping all
other ~LaTeX special characters (skipping the
already escaped backslashes.

The final step is to remove the escaped backslashes
 for some reason - probably related to the noweb
macros, the math mode backslash markers don't work?
They should but I don't know why they don't.

So we simply replace the flagged backslash markers
with an escaped backslash sequence. 8 backslashes
seems to do the job (translating to 4 real backslashes)
")

(defun elnw-latex-escape-specials (string)
  (with-temp-buffer
    (insert string)
    (elnw-latex-escape (point-min) (point-max))
    (buffer-string)))

(defun elnw-latex-escape (start end)
  "LaTeX-escape special characters in the region from START to END."
  (save-excursion
    (save-restriction
      (narrow-to-region start end)
      (goto-char (point-min))
      (save-excursion
        (while (re-search-forward "\\\\" nil t)
          (replace-match "$\\backslash$" nil t)))
      (save-excursion
        (while (re-search-forward "\\([#%&~$_^{}]\\)" nil t)
          ;; Don't re-escape our escaped backslashes.
          (cond
	   ((and (equal (char-before) ?\$)
		 (looking-at (regexp-quote "\\backslash$")))
	    (goto-char (match-end 0)))
	   ((equal (char-before) ?^)
	    (replace-match "\\\\^{}"))
	   ((equal (char-before) ?~)
	    (replace-match "\\\\~{}"))
	   (t
            (replace-match "\\\\\\1"))))
	(goto-char (point-min))
	(while (search-forward "\$\\backslash\$" nil t)
	  (replace-match "\\\\\\\\"))))))


(noweb-doc t
"
~subsubsection{elnw-writeb}
The function elnw-writeb simply writes
a message to some buffer without
disturbing the current buffer positioning.")
(defun elnw-writeb (outb msg)
  (save-excursion
    (with-current-buffer outb
      (goto-char (point-max))
      (insert msg)
      (point))))


(noweb-doc t
"
~subsubsection{elnw-find-defintions}
This functions does the auto-indexing
of lisp defintions, finding the
items that need to be def'ed in the noweb
file at file conversion time (going from
~extnwel to ~extel file).


It picks up a list of all definitions
by scanning the lisp code
between points start and end in the current
buffer (definitions like -
defun, defconst, defvar, .... there are
many such things). It is fairly simple
since the font-lock on the buffer has already 
done the hard work. 

It simply searches the buffer based on
'face' property changes. If at each change
it detects a face property which is in the list
elnw-faces-defining-things then it picks up the
s-expression at that point and adds it to the list
to be returned.

Right now font-lock-variable-name-face and
font-lock-function-name-face seem to pick up
everything.
")
(defun elnw-find-definitions ()
"Find all definitions by virtue of font lock faces
in the current buffer between start and end and
return all the names of things defined in a list."
  (let ((face) (thing) (defs) (p1) (p2) (p3))
    (goto-char (point-min))
    (while (not (eobp))
      (setq face (get-text-property (point) 'face))
      (cond
       ((member face elnw-faces-defining-things)
	(setq thing (thing-at-point 'sexp))
	(cond
	 ((and (stringp thing)
	       (= (length (split-string thing)) 1))
	  (setq defs (append defs 
			     (list thing)))))))
      (setq p1 (next-property-change (point)))
      (if p1 (goto-char p1) (goto-char (point-max))))
    defs))

(noweb-doc t
"The function elnw-dump-defs simply
takes the list of definitions created
by the preceding function and turns
it into a proper noweb def line and inserts
it into theoutput buffer.")

(defun elnw-dump-defs (outb defs)
"Write a line in noweb format containing the
set of things defined in the current block as
determined by font-lock"
   (let ((stuff))
     (cond
      (defs
	(setq stuff (mapconcat 'concat defs " "))
	(save-excursion
	  (with-current-buffer outb
	    (insert (concat "@ %def " stuff "\n"))))))))

(noweb-doc t
"subsubsection{Replace Latex Macro Marker}
This function is responsible for taking
any alternate marker for ~LaTeX macro start 
the user specified by placing an 
~verb+@@macro-marker [new-marker]+ line
in a doc chunk.

This new marker needs to be replaced with
the real latex backslah marker when we 
convert our comments to real latex. 

It scans the input string for any occurrences
of the form ~verb+@@marker x+. The last such
occurrence (normally there would only be 1 in an
entire file, in the block defines the new
replacement marker which stays in place
until set to something else.

After doing this it determines whether in
fact any replacement needs to be done. If the 
standard backslash is in place then it does nothing.
 
The business of replacement is a little convoluted.
We first need to find all doubles of our replacement
marker (meaning we want 1 literal instance), and 
flag those. The we replace all single instances
with a backslash, and finally we replace the
escaped instances with a single literal instance
of the replacement marker. If the replacement
marker is a latex special then it must be escaped
for latex when we do these last insertions.
")

(defun elnw-replace-latex-macro-marker (in)
"Replace all occurrences of ~ with backslash
in string IN"
  (let ((out in)
	(marker) (real-marker)
	(place-mark "===XyZAbC===")
	(rep-marker))
    ;; see if we have a new marker in the input
    ;; if so -then set the new marker first
    (while (string-match "^@@macro-marker \\(.*\\)\n" out)
      (let ((new-marker (match-string 1 out)))      
	(setf (car ELNW-TEX-MACRO-START) new-marker)
	(setq out (replace-match "" t t out))))
    ;; init the local markers
    (setq marker (car ELNW-TEX-MACRO-START)
	  real-marker (nth 1 ELNW-TEX-MACRO-START))
    (cond
     ;; now if we need to do any replacements
     ((not (string= marker real-marker))
      ;; first flag escaped replacement markers
      ;; (2 markers in succession)
      (while (string-match (concat marker marker) out)
	(setq out
	      (replace-match 
	       place-mark t t out)))
      ;; now replace all non-escaped markers 
      ;; with the correct marker
      (while (string-match marker out)
	(setq out (replace-match real-marker
				 t t out)))
      ;; finally replace the flagged escaped markers
      ;; with a single marker
      (setq rep-marker (elnw-latex-escape-specials marker))
      (while (string-match place-mark out)
	(setq out (replace-match rep-marker t t out)))))
    out))
	

(noweb-doc "elnw.el"
"
~subsubsection{elnw-check-file}
The function ~emph{elnw-check-file} is a utility
function used to take in an input file of either
extension ~extnwel and transform it to a pure noweb
~extnw file by invoking elnw-el-to-nw. If the file
is already in that final format and it exists
then no additional processing takes place.

The optional argument ~emph{no-create} disables
any conversion and the only test is input file
existence.

On success the function returns the name of the
converted file (or the original if no-create is
flagged) and nil is returned on failure.
")

(defun elnw-check-file (fn &optional no-create)
"Checks if FN has an extension .nw if it does
return FN. If FN has extension NW.EL then create
the NW version using elfn above and return 
the NW file name."
  (let* ((ext1 (file-name-extension fn))
	 (fn1 (file-name-sans-extension fn))
	 (ext2 (file-name-extension fn1))
	 (dir (file-name-directory fn))
	 (roots1) (root) (out) (errors 0)
	 (tmp) (res))
    (cond
     ((and (stringp ext1)
	   (string= (downcase ext1) "nw"))
      fn)
     ((and (stringp ext1)
	   (string= (downcase ext1) "el")
	   (stringp ext2)
	   (string= (downcase ext2) "nw"))
      (cond
       ((and no-create (not (file-exists-p fn1)))
	(elnw-log (format "No such file: %s\n" fn1))
	(setq fn1 nil))
       ((and no-create (file-exists-p fn1))
	(elnw-log (format "Using existing nw file: %s\n" fn1)))
       (t
	(setq errors (elnw-el-to-nw fn fn1))
	(cond
	 ((= errors -1)
	  (setq fn1 nil)
	  (elnw-log 
	   (format "Failed to open input file: %s\n" fn)))
	 ((= errors -2)
	  (elnw-log 
	   (format "Failed to create output file:%s\n" fn1))
	  (setq fn1 nil))
	 ((> errors 0)
	  (elnw-log
	   (format "Found syntax errors in input file: %s\n" fn))
	  (setq fn1 nil))
	 ;; now we we check for any differences
	 (t
	  (elnw-log 
	   (format "\nSuccessfully generated %s from %s\n" 
		   fn1 fn))))))
      fn1)
     (t
      (elnw-log 
       (format "\nBad noweb file extension: %s\n" fn))
      nil))))

(noweb-doc "elnw.el"
"
~subsubsection{Some Minor Utility Functions}
The function ~emph{elnw-log} prints a compile
message to the compile-log buffer if we are 
running interactively - otherwise - in
non-interactive mode it prints a message to stderr.
")

(defun elnw-log (msg)
  (cond
   (noninteractive
    (princ msg))
   (t
    (save-excursion
      (let ((cl (get-buffer-create "*elnw-Log*")))
	(with-current-buffer cl
	  (goto-char (point-max))
	  (insert msg)
	  (insert "\n")))))))

(noweb-doc t
"~emph{elnw-get-comment-marker} finds
out what we are currently using as a single
line comment flag when we need to pass certain
lines in place in and out of a file requiring
fontification.")

(defun elnw-get-comment-marker (cur-marker lang)
"Get the single line comment marker based
on any cur-marker value explicitly stated or
based on the current language explicitly specified
or retun an empty string if nothing found."
  (cond
   ((stringp cur-marker)
    cur-marker)
   (t
    (let ((lookup (nth 2 (assoc lang elnw-lang-flags))))
      (cond
       ((stringp lookup) lookup)
       (t ""))))))

(noweb-doc t
"~emph{elnw-fixup-tex} is called to insert
a ~LaTeX preamble and postfix on a tex file
generated with noweave and the delay option
just prior to invoking latex and pdflatex.")

(defun elnw-fixup-tex (fn &optional xdvi-flag)
"Take tex file FN generated with noweave 
and -delay (with latex colorization) and
add the appropriate preamble required
for processing the noweb style latex file."
  (cond
   ((file-exists-p fn)
    (save-excursion 
      (find-file fn)
      (goto-char (point-min))
      (cond
       ((search-forward "\\begin{document}" nil t)
	(forward-line)
	(delete-region (point-min) (point))
	(goto-char (point-min))))
      (if xdvi-flag
	(insert ELNW-TEX-PREAMBLE-XDVI)
	(insert ELNW-TEX-PREAMBLE))
      (cond
       ((not (search-forward "\\end{document}" nil t))
	(goto-char (point-max))
	(insert ELNW-TEX-FINAL)))
      (basic-save-buffer)
      (kill-buffer nil)
      t))
   (t nil)))

(defun elnw-read-one-line (&optional inb)
  (cond
   ((not inb)
    (car (split-string (read-string "") "\n")))
   ((bufferp inb)
    (with-current-buffer inb
      (let ((l))
	(cond
	 ((eobp) nil)
	 (t
	  (setq l (car (split-string (thing-at-point 'line) "\n")))
	  (forward-line)
	  l)))))
   (t nil)))

(defun elnw-write-one-line (out l)
"Write one line to buffer OUT or if OUT is nil
princ it to stdout"
  (cond
   ((bufferp out)
    (save-excursion
      (with-current-buffer out
	(goto-char (point-max))
	(insert l))))
   (t
    (princ l))))

(noweb-doc nil
"
@@language sh
~section{Shell Scripts}
The following shell scripts are used to
invoke emacs functionality in the 
~emph{elnw.el} package from the command
line.

~subsection{~emph{elnw-fontify-filter}}
~emph{elnw-fontify-filter}
can be invoked in the ~emph{noweave}
filter chain. If present there it will color
source code chunks (in any language) according 
to the font-lock rules in place in the user's
own .~,emacs file.

The script passes the input and output filenames
to emacs via environment variables ELIN and ELOUT.

This is necessary since in batch mode we can only
call functions that take no arguments.
")

(noweb-other-code "elnw-fontify-filter"
"
#!/bin/sh
export ELIN=$1
export ELOUT=$2
ARGS=\"--batch --load=/usr/local/share/emacs/site-lisp/elnw.elc --funcall=elnw-fontify-filter\" 
emacs $ARGS
") 

(noweb-doc nil 
"
~subsection{~emph{el to nw File Converter}}

The following script invokes the el to nw converter.
It takes an input and output (optional) filename
argument. It transforms the emacs lisp format noweb
file into pure noweb format. It also auto-indexes all
lisp function, variable, macro, constant, ... definitions
and inserts the appropriate @ ~%def lines in the noweb 
output, for all root sections found that show a ~extel
file extension. When the elisp source files are
untangled - all the in-line documentation will have been
removed from the resultant lisp source file.
If no output file is specified in the command line
it defaults to the input file with the trailing ~extel
extension removed.
")

(noweb-other-code "elnw"
"
#!/bin/sh
export ELIN=$1
export ELOUT=$2
ARGS=\"--batch --load=/usr/local/share/emacs/site-lisp/elnw.elc --funcall=elnw-convert\" 
emacs $ARGS
")

(noweb-doc nil
"~subsection{~emph{elnw-compile} Byte Compile all Roots}
The script ~emph{elnw-compile} takes as an argument
the input file which may have  either an ~extnwel or ~extel 
file extension. If necessary it runs the converter to
transform the input to pure noweb format. It then examines
all root code names and finds those with a ~extel extension.
These files are all extracted with notangle and byte-compiled.
The resultant ~extel and ~extel~,c files are left in the
same directory as the input file.
")

(noweb-other-code "elnw-compile"
"
#!/bin/sh
export ELIN=$1
ARGS=\"--batch --load=/usr/local/share/emacs/site-lisp/elnw.elc --funcall=elnw-compile\" 
emacs $ARGS
")

(noweb-doc nil
"~subsection{~emph{elnw-gen-docs} Generate Docs}
This script invokes the automated doc generator
which will produce the documented literate source
code in pdf and dvi format from a single file
 (argument 1) which may be either a ~extnwel or ~extnw
format file. All the source code in the document will
be emacs font lock colored, indexed and hyper-ref'ed. 
The optional second specifies a directory either relative
to the infile directory or an absolute path, in which
to store the documents.
")

(noweb-other-code "elnw-gen-docs"
"
#!/bin/sh
export ELIN=$1
export ELOUT=$2
ARGS=\"--batch --load=/usr/local/share/emacs/site-lisp/elnw --funcall=elnw-gen-docs\" 
emacs $ARGS
")
